
-- This function checks any loaded data from save to ensure it is compatible with the current version
function UpdateData()

	if _BOOKMARKS.__VERSION == nil or _BOOKMARKS.__VERSION < V_BOOKMARKS then
		_BOOKMARKS.__VERSION = nil;
		_BOOKMARKS = UpdateLoadedBookmarks(_BOOKMARKS);
	end

	if _BUILDS.__VERSION == nil or _BUILDS.__VERSION < V_BUILDS then
		_BUILDS.__VERSION = nil;
		_BUILDS = UpdateLoadedBuilds(_BUILDS);
	end

	if _ITEMS.__VERSION == nil or _ITEMS.__VERSION < V_ITEMS then
		_ITEMS.__VERSION = nil;
		_ITEMS = UpdateLoadedItems(_ITEMS);
	end

	if SETTINGS.__VERSION == nil or SETTINGS.__VERSION < V_SETTINGS then
		SETTINGS.__VERSION = nil;
		SETTINGS = UpdateLoadedSettings(SETTINGS);
	end

	_SEARCHHISTORY.__VERSION = nil;	-- Not checked at present
	_BOOKMARKS.__VERSION = nil;
	_BUILDS.__VERSION = nil;
	_ITEMS.__VERSION = nil;
	SETTINGS.__VERSION = nil;

	-- remove functions to garbage as not needed again
	UpdateLoadedBookmarks = nil;
	UpdateLoadedSettings = nil;
	UpdateLoadedBuilds = nil;
	UpdateLoadedItems = nil;
end


function UpdateLoadedItems(_loadedItems)

	if _loadedItems == nil or type(_loadedItems) ~= 'table' then return {} end;
	local _TULKAS = {};

	-- Value tables for conversion.
	_TULKAS["RARITY"] =
	{
		["White"] = 1;
		["Yellow"] = 2;
		["Purple"] = 3;
		["Teal"] = 4;
		["Orange"] = 5;
	};


	_TULKAS["SLOT"] =
	{
		["Ear"] = 1;
		["Neck"] = 2;
		["Pocket"] = 3;
		["Wrist"] = 4;
		["Finger"] = 5;
		["Head"] = 6;
		["Shoulder"] = 7;
		["Back"] = 8;
		["Chest"] = 9;
		["Hand"] = 10;
		["Leg"] =11;
		["Feet"] = 12;
		["Melee Weapon"] = 13;
		["Shield"] = 14;
		["Ranged Weapon"] = 15;
		["Rune-stone"] = 16;
	};


	_TULKAS["ARMORTYPE"] =
	{
		["Light"] = 1;
		["Medium"] = 2;
		["Heavy"] = 3;
	};


	_TULKAS["WEAPONTYPE"] =
	{

		["One-handed Axe"] = 1;
		["Two-handed Axe"] = 2;
		["One-handed Club"] = 3;
		["Two-handed Club"] = 4;
		["Dagger"] = 5;
		["Halberd"] = 6;
		["One-handed Hammer"] = 7;
		["Two-handed Hammer"] = 8;
		["One-handed Mace"] = 9;
		["Spear"] = 10;
		["Staff"] = 11;
		["One-handed Sword"] = 12;
		["Two-handed Sword"] = 13;

		["Light"] = 14;
		["Heavy"] = 15;
		["Warden"] = 16;

		["Bow"] = 17;
		["Crossbow"] = 18;
		["Javelin"] = 19;
	};


	_TULKAS["DAMAGETYPE"] =
	{
		["Common"] = 1;
		["Beleriand"] = 2;
		["Westernesse"] = 3;
		["Ancient Dwarf-make"] = 4;
		["Fire"] = 5;
		["Light"] = 6;
		["Lightning"] = 7;
		["Frost"] = 8;
	};


	_TULKAS["CALLING"] =
	{
		["Burglar"] = 40;
		["Captain"] = 24;
		["Champion"] = 172;
		["Guardian"] = 23;
		["Hunter"] = 162;
		["LoreMaster"] = 185;
		["Minstrel"] = 31;
		["RuneKeeper"] = 193;
		["Warden"] = 194;
	};


	_TULKAS["STATS"] =
	{
		["Maximum Morale"] = 1;
		["in-Combat Morale Regen"] = 2;
		["non-Combat Morale Regen"] = 3;

		["Maximum Power"] = 4;
		["in-Combat Power Regen"] = 5;
		["non-Combat Power Regen"] = 6;

		["Might"] = 7;
		["Agility"] = 8;
		["Vitality"] = 9;
		["Will"] = 10;
		["Fate"] = 11;

		["Critical Rating"] = 12;
		["Finesse Rating"] = 13;
		["Physical Mastery Rating"] = 14;
		["Tactical Mastery Rating"] = 15;

		["Resistance Rating"] = 16;
		["Critical Defence"] = 17;
		["Incoming Healing Rating"] = 18;

		["Block Rating"] = 19;
		["Parry Rating"] = 20;
		["Evade Rating"] = 21;

		["Melee Defence"] = 22;
		["Ranged Defence"] = 23;
		["Tactical Defence"] = 24;

		["Physical Mitigation"] = 25;
		["Tactical Mitigation"] = 26;

		["Audacity"] = 27;
		["Stealth Level"] = 28;
		["Tactical Critical Multiplier"] = 29;
	};

	local _returnTable = {};

	for k,v in pairs(_loadedItems) do

		local itemID = k;
		if type(itemID) == 'string' then itemID = string.gsub(itemID,"#","") end;
		itemID = tonumber(itemID);

		if itemID > 10000 and type(v) == 'table' then

			_returnTable[itemID] = {};

			if v.Name ~= nil then
				_returnTable[itemID][1]={[1]="";[2]="";[3]="";[4]="";};
				_returnTable[itemID][1][LANGID] = v.Name;
			end

			if v.Level ~= nil then
				_returnTable[itemID][2] = string.gsub(v.Level,"#","");
				_returnTable[itemID][2] = tonumber(_returnTable[itemID][2]);
			end

			if v.ItemLevel ~= nil then
				_returnTable[itemID][3] = string.gsub(v.ItemLevel,"#","");
				_returnTable[itemID][3] = tonumber(_returnTable[itemID][3]);
			end

			if v.Color ~= nil then _returnTable[itemID][4] = _TULKAS.RARITY[v.Color] end;
			if v.Slot ~= nil then _returnTable[itemID][5] = _TULKAS.SLOT[v.Slot] end;

			if v.Stats ~= nil and type(v.Stats)=='table' then
				_returnTable[itemID][6] = {};
				for statName,statVal in pairs(v.Stats) do
					if _TULKAS.STATS[statName] ~= nil then
						_returnTable[itemID][6][_TULKAS.STATS[statName]] = string.gsub(statVal,"#","");
						_returnTable[itemID][6][_TULKAS.STATS[statName]] = tonumber(_returnTable[itemID][6][_TULKAS.STATS[statName]]);
					end
				end
				if next(_returnTable[itemID][6]) == nil then
					_returnTable[itemID][6] = nil;
				end
				if v.Stats.Armour ~= nil and v.Stats.Armour ~= "" then
					_returnTable[itemID][8] = string.gsub(v.Stats.Armour,"#","");
					_returnTable[itemID][8] = tonumber(_returnTable[itemID][8]);
				end
				if v.Stats.Damage ~= nil then
					if v.Stats.Damage.MinDMG ~= nil and v.Stats.Damage.MinDMG ~= "" then
						_returnTable[itemID][10] = string.gsub(v.Stats.Damage.MinDMG,"#","");
						_returnTable[itemID][10] = tonumber(_returnTable[itemID][10]);
					end

					if v.Stats.Damage.MaxDMG ~= nil and v.Stats.Damage.MaxDMG ~= "" then
						_returnTable[itemID][11] = string.gsub(v.Stats.Damage.MaxDMG,"#","");
						_returnTable[itemID][11] = tonumber(_returnTable[itemID][11]);
					end

					if v.Stats.Damage.DPS ~= nil and v.Stats.Damage.DPS ~= "" then
						_returnTable[itemID][12] = string.gsub(v.Stats.Damage.DPS,"#","");
						_returnTable[itemID][12] = tonumber(_returnTable[itemID][12]);
					end

					if v.Stats.Damage.TypeDMG ~= nil and v.Stats.Damage.TypeDMG ~= "" then _returnTable[itemID][13] = _TULKAS.DAMAGETYPE[v.Stats.Damage.TypeDMG] end;
				end
			end

			if v.Type ~= nil and v.Type ~= "" and v.Type ~= "Cloak" then _returnTable[itemID][7] = _TULKAS.ARMORTYPE[v.Type] end;

			if v.SubSlot ~= nil and v.SubSlot ~= "" then _returnTable[itemID][9] = _TULKAS.WEAPONTYPE[v.SubSlot] end;

			if v.Class ~= nil and v.class ~= "" then _returnTable[itemID][14] = _TULKAS.CALLING[v.Class] end;
		end
	end

	return _returnTable;
end


function UpdateLoadedBookmarks(_loadedTable)

	if _loadedTable == nil or type(_loadedTable) ~= 'table' then return {} end;
	local _returnTable = {};

	for k,v in pairs (_loadedTable) do

		if type (v) == 'table' then
			local key = string.gsub(k,"#","");
			key = tonumber(key);
			table.insert(_returnTable,key);
		else
			table.insert(_returnTable,v);
		end
	end

	return _returnTable;
end


function UpdateLoadedBuilds(_loadedBuilds)

	if _loadedBuilds == nil or type(_loadedBuilds) ~= 'table' then return {} end;
	local _returnTable = {};

	for k,v in pairs (_loadedBuilds) do
		local _newTable = {};

		if v["Earring1"] ~= nil and type( v["Earring1"] ) == 'table' then
			for kk,vv in pairs ( v["Earring1"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[13] = key;
				if _newTable[13] < 100000 then _newTable[13] = nil end;
			end
		end

		if v["Earring2"] ~= nil and type( v["Earring2"] ) == 'table' then
			for kk,vv in pairs ( v["Earring2"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[14] = key;
				if _newTable[14] < 100000 then _newTable[14] = nil end;
			end
		end

		if v["Necklace"] ~= nil and type( v["Necklace"] ) == 'table' then
			for kk,vv in pairs ( v["Necklace"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[10] = key;
				if _newTable[10] < 100000 then _newTable[10] = nil end;
			end
		end

		if v["Bracelet1"] ~= nil and type( v["Bracelet1"] ) == 'table' then
			for kk,vv in pairs ( v["Bracelet1"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[8] = key;
				if _newTable[8] < 100000 then _newTable[8] = nil end;
			end
		end

		if v["Bracelet2"] ~= nil and type( v["Bracelet2"] ) == 'table' then
			for kk,vv in pairs ( v["Bracelet2"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[9] = key;
				if _newTable[9] < 100000 then _newTable[9] = nil end;
			end
		end

		if v["Ring1"] ~= nil and type( v["Ring1"] ) == 'table' then
			for kk,vv in pairs ( v["Ring1"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[11] = key;
				if _newTable[11] < 100000 then _newTable[11] = nil end;
			end
		end

		if v["Ring2"] ~= nil and type( v["Ring2"] ) == 'table' then
			for kk,vv in pairs ( v["Ring2"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[12] = key;
				if _newTable[12] < 100000 then _newTable[12] = nil end;
			end
		end

		if v["Pocket"] ~= nil and type( v["Pocket"] ) == 'table' then
			for kk,vv in pairs ( v["Pocket"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[15] = key;
				if _newTable[15] < 100000 then _newTable[15] = nil end;
			end
		end

		if v["Head"] ~= nil and type( v["Head"] ) == 'table' then
			for kk,vv in pairs ( v["Head"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[1] = key;
				if _newTable[1] < 100000 then _newTable[1] = nil end;
			end
		end

		if v["Shoulder"] ~= nil and type( v["Shoulder"] ) == 'table' then
			for kk,vv in pairs ( v["Shoulder"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[6] = key;
				if _newTable[6] < 100000 then _newTable[6] = nil end;
			end
		end

		if v["Back"] ~= nil and type( v["Back"] ) == 'table' then
			for kk,vv in pairs ( v["Back"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[7] = key;
				if _newTable[7] < 100000 then _newTable[7] = nil end;
			end
		end

		if v["Chest"] ~= nil and type( v["Chest"] ) == 'table' then
			for kk,vv in pairs ( v["Chest"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[2] = key;
				if _newTable[2] < 100000 then _newTable[2] = nil end;
			end
		end

		if v["Gloves"] ~= nil and type( v["Gloves"] ) == 'table' then
			for kk,vv in pairs ( v["Gloves"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[4] = key;
				if _newTable[4] < 100000 then _newTable[4] = nil end;
			end
		end

		if v["Legs"] ~= nil and type( v["Legs"] ) == 'table' then
			for kk,vv in pairs ( v["Legs"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[3] = key;
				if _newTable[3] < 100000 then _newTable[3] = nil end;
			end
		end

		if v["Boots"] ~= nil and type( v["Boots"] ) == 'table' then
			for kk,vv in pairs ( v["Boots"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[5] = key;
				if _newTable[5] < 100000 then _newTable[5] = nil end;
			end
		end

		if v["PrimaryWeapon"] ~= nil and type( v["PrimaryWeapon"] ) == 'table' then
			for kk,vv in pairs ( v["PrimaryWeapon"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[16] = key;
				if _newTable[16] < 100000 then _newTable[16] = nil end;
			end
		end

		if v["SecondaryWeapon"] ~= nil and type( v["SecondaryWeapon"] ) == 'table' then
			for kk,vv in pairs ( v["SecondaryWeapon"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[17] = key;
				if _newTable[17] < 100000 then _newTable[17] = nil end;
			end
		end

		if v["RangedWeapon"] ~= nil and type( v["RangedWeapon"] ) == 'table' then
			for kk,vv in pairs ( v["RangedWeapon"] ) do
				local key = string.gsub(kk,"#","");
				key = tonumber(key);
				_newTable[18] = key;
				if _newTable[18] < 100000 then _newTable[18] = nil end;
			end
		end

		_newTable[19] = _LABELS[27][LANGID];

		if tonumber(k) == 1 then
			if SETTINGS.BUILD1 ~= nil then _newTable[19] = SETTINGS.BUILD1 end;
		elseif tonumber(k) == 2 then
			if SETTINGS.BUILD2 ~= nil then _newTable[19] = SETTINGS.BUILD2 end;
		elseif tonumber(k) == 3 then
			_newTable[19] = _LABELS[13][LANGID];
		end

		if tonumber(k) == 3 then
			table.insert(_returnTable,1,_newTable);
		else
			table.insert(_returnTable,_newTable);
		end
	end

	return _returnTable;
end


function UpdateLoadedSettings(_loadedSettings)

	_loadedSettings["LANGUAGE"] = nil;
	_loadedSettings["MSGCOLOR"] = nil;
	_loadedSettings["PLUGINNAMEINLINKS"] = nil;
	_loadedSettings["CHATCHANNEL"] = nil;
	_loadedSettings["CHANNEL"] = nil;
	_loadedSettings["MAXRESULTS"] = nil;
	_loadedSettings["BUILD1"] = nil;
	_loadedSettings["BUILD2"] = nil;
	_loadedSettings["COMPAREWIN"] = nil;
	_loadedSettings["MYITEMSWIN"] = nil;

	if _loadedSettings["MAINWIN"] ~= nil then _loadedSettings.MAINWIN["VISIBLE"] = nil end;

	return _loadedSettings;
end


