
-- This function creates the control boxes used to show each item in the search results, along with the item info.
function GetItemInfoDisplay(itemID)

	if itemID == nil then return nil end;

	local itemHex = Utils.TO_HEX(itemID);
	local _itemInfo = _ITEMSDB[itemID];
	local validID = false;

	local containerWidth = ITEMINFOWIDTH;
	local containerHeight = 0;

	local cItemContainer = NewItemContainer(containerWidth,containerHeight)
	--cItemContainer:SetBackColor(Turbine.UI.Color(0,0,0));

	local cBack = Turbine.UI.Control();
	cBack:SetParent(cItemContainer);
	cBack:SetSize(containerWidth-8,3);
	cBack:SetPosition(2,2);
	cBack:SetBackColor(Turbine.UI.Color(0.08,0.08,0.085));

	-- ITEM INSPECT CONTROL ------------------------------------------------------------------------------------------------
	local cItemInspect = NewItemInfo(itemID);
	cItemInspect:SetParent(cItemContainer);
	cItemInspect:SetPosition(0,cBack:GetTop()+cBack:GetHeight());
	if cItemInspect:GetItemInfo() ~= nil then validID = true end;

	-- ITEM NAME LABEL -----------------------------------------------------------------------------------------------------
	local lblItemName = Turbine.UI.Label();
	lblItemName:SetParent(cItemContainer);
	lblItemName:SetSize(containerWidth-50,40);
	lblItemName:SetPosition(42,cItemInspect:GetTop());
	lblItemName:SetForeColor(_QUALITYCOLORS[_itemInfo[4]]);
	lblItemName:SetFont(_FONTS[3]);
	lblItemName:SetOutlineColor(Turbine.UI.Color(0,0,0));
	lblItemName:SetFontStyle(Turbine.UI.FontStyle.Outline);
	lblItemName:SetMouseVisible(false);
	lblItemName:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);

	if _itemInfo[1][LANGID] == nil or _itemInfo[1][LANGID] == "" then	-- If item name doesn't exist in the player's language then default to english
		lblItemName:SetText(_itemInfo[1][1]);
	else
		lblItemName:SetText(_itemInfo[1][LANGID]);
	end

	-- ITEM DETAILS LISTBOX -----------------------------------------------------------------------------------------------
	local lstItemInfo = Turbine.UI.ListBox();
	lstItemInfo:SetParent(cItemContainer);
	lstItemInfo:SetPosition(3,lblItemName:GetTop()+lblItemName:GetHeight()+2);
	lstItemInfo:SetSize(containerWidth-6,0);

	containerHeight = lstItemInfo:GetTop();

	-- ITEM SLOT -----------------------------------------------------------------------------------------------------------
	if _itemInfo[5] >= 1 and _itemInfo[5] <= 5 then 	-- only display slot when jewellery
		local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblItemInfo:SetText(_TABLES.SLOT[_itemInfo[5]][LANGID]);
		lstItemInfo:AddItem(lblItemInfo);
		containerHeight = containerHeight + lblItemInfo:GetHeight();
	end

	-- ARMOUR TYPE (Light/Med/Heavy) ----------------------------------------------------------------------------------------
	if _itemInfo[7] ~= nil then
		local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblItemInfo:SetText(_TABLES.ARMORTYPE[_itemInfo[7]][LANGID]);
		if _itemInfo[7] > _TABLES.ARMOURDEFAULTS[PLAYERCLASS] then lblItemInfo:SetForeColor(Turbine.UI.Color.Red) end;
		lstItemInfo:AddItem(lblItemInfo);
		containerHeight = containerHeight + lblItemInfo:GetHeight();
	end

	-- WEAPON TYPE ----------------------------------------------------------------------------------------------------------
	if _itemInfo[9] ~= nil then
		local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblItemInfo:SetText(_TABLES.WEAPONTYPE[_itemInfo[9]][LANGID]);
		lstItemInfo:AddItem(lblItemInfo);
		containerHeight = containerHeight + lblItemInfo:GetHeight();
	end

	-- MIN/MAX DAMAGE & TYPE ------------------------------------------------------------------------------------------------
	if _itemInfo[10] ~= nil and _itemInfo[11] ~= nil and _itemInfo[13] ~= nil then
		local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblItemInfo:SetText(tostring(_itemInfo[10]) .. " - " .. tostring(_itemInfo[11]) .. " " ..  _TABLES.DAMAGETYPE[_itemInfo[13]][LANGID]);
		lstItemInfo:AddItem(lblItemInfo);
		containerHeight = containerHeight + lblItemInfo:GetHeight();
	end

	-- DPS ------------------------------------------------------------------------------------------------------------------
	if _itemInfo[12] ~= nil then
		local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblItemInfo:SetText(tostring(_itemInfo[12]) .. " " .. _LABELS[11][LANGID]);
		lblItemInfo:SetForeColor(Turbine.UI.Color.Khaki);
		lstItemInfo:AddItem(lblItemInfo);
		containerHeight = containerHeight + lblItemInfo:GetHeight();
	end

	-- Empty slot (break)
	local cLineBreak =NewItemContainer(lstItemInfo:GetWidth()-3);
	lstItemInfo:AddItem(cLineBreak);
	containerHeight = containerHeight + cLineBreak:GetHeight();

	-- ARMOUR RATING --------------------------------------------------------------------------------------------------------
	if _itemInfo[8] ~= nil then
		local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblItemInfo:SetText(Utils.comma_value(_itemInfo[8]) .. " " .. _LABELS[12][LANGID]);
		lblItemInfo:SetForeColor(Turbine.UI.Color.Khaki);
		lstItemInfo:AddItem(lblItemInfo);
		containerHeight = containerHeight + lblItemInfo:GetHeight();
	end

	-- STATS ----------------------------------------------------------------------------------------------------------------
	if _itemInfo[6] ~= nil then
		for k,v in pairs(_itemInfo[6]) do
			local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
			lblItemInfo:SetText(GetStatPrefix(k) .. Utils.comma_value(v) .. GetStatSuffix(k) .. " " .. _TABLES.STATS[k][LANGID]);
			lblItemInfo:SetForeColor(Turbine.UI.Color.Lime);
			lstItemInfo:AddItem(lblItemInfo);
			containerHeight = containerHeight + lblItemInfo:GetHeight();
		end
	end

	-- Empty slot (break)
	local cLineBreakTwo =NewItemContainer(lstItemInfo:GetWidth()-3);
	lstItemInfo:AddItem(cLineBreakTwo);
	containerHeight = containerHeight + cLineBreakTwo:GetHeight();

	-- MIN LEVEL ------------------------------------------------------------------------------------------------------------
	if _itemInfo[2] ~= nil and _itemInfo[2] ~= 0 then
		local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblItemInfo:SetText(_LABELS[6][LANGID] .. " " .. tostring(_itemInfo[2]));
		if PLAYERLEVEL < _itemInfo[2] then lblItemInfo:SetForeColor(Turbine.UI.Color.Red) end;
		lstItemInfo:AddItem(lblItemInfo);
		containerHeight = containerHeight + lblItemInfo:GetHeight();
	end

	-- CLASS ----------------------------------------------------------------------------------------------------------------
	if _itemInfo[14] ~= nil then
		local lblItemInfo = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblItemInfo:SetText(_TABLES.ITEMINFO[14][LANGID] .. ": " .. _TABLES.CLASS[_itemInfo[14]][LANGID]);
		if PLAYERCLASS ~= _itemInfo[14] then lblItemInfo:SetForeColor(Turbine.UI.Color.Red) end;
		lstItemInfo:AddItem(lblItemInfo);
		containerHeight = containerHeight + lblItemInfo:GetHeight();
	end

	-- Empty slot (break)
	local cLineBreakThree =NewItemContainer(lstItemInfo:GetWidth()-3);
	lstItemInfo:AddItem(cLineBreakThree);
	containerHeight = containerHeight + cLineBreakThree:GetHeight();

	-- Set ------------------------------------------------------------------------------------------------------------------
	if _itemInfo[15] ~= nil then
		-- Set Name
		local lblSetName = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
		lblSetName:SetForeColor(Turbine.UI.Color.Orange);
		lblSetName:SetText(_itemInfo[15][1][LANGID]);

		if _itemInfo[15][1][LANGID] == nil or _itemInfo[15][1][LANGID] == "" then	-- If item name doesn't exist in the player's language then default to english
			lblSetName:SetText(_itemInfo[15][1][1]);
		else
			lblSetName:SetText(_itemInfo[15][1][LANGID]);
		end

		-- if length too long then clip and set tooltip.
		if lblSetName:GetTextLength() > 35 then
			lblSetName:SetText(string.sub(lblSetName:GetText(),1,33) .. "...");
			lblSetName:SetToolTip(_itemInfo[15][1][LANGID]);
		end
		lblSetName.MouseClick = function()	-- set search results link when clicked
			if _itemInfo[15][2] ~= nil then
				PrepareResults(_itemInfo[15][2]);
			end
		end
		lstItemInfo:AddItem(lblSetName);
		containerHeight = containerHeight + lblSetName:GetHeight();
		-- Set Bonus
		if _itemInfo[15][3] ~= nil and type(_itemInfo[15][3]) == 'table' then
			for k,v in pairs (_itemInfo[15][3]) do -- loops through 'number equipped' tables
				local lblPiecesEquipped = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
				lblPiecesEquipped:SetText(tostring(k) .. " " .. _LABELS[13][LANGID] .. ":");
				lblPiecesEquipped:SetForeColor(Turbine.UI.Color.LimeGreen);
				lstItemInfo:AddItem(lblPiecesEquipped);
				containerHeight = containerHeight + lblPiecesEquipped:GetHeight();
				-- loop through equipped pieces bonuses
				if type(v) == 'table' then
					for kk,vv in pairs (v) do
						local lblSetBonus = NewItemInfoLabel(lstItemInfo:GetWidth()-3);
						lblSetBonus:SetText("  " .. GetStatPrefix(kk) .. Utils.comma_value(vv) .. GetStatSuffix(kk) .. " " .. _TABLES.STATS[kk][LANGID]);
						lblSetBonus:SetForeColor(Turbine.UI.Color.DarkGray);
						lstItemInfo:AddItem(lblSetBonus);
						containerHeight = containerHeight + lblSetBonus:GetHeight();
					end
				end
			end
		end

	-- Empty slot (break)
	local cLineBreakFour =NewItemContainer(lstItemInfo:GetWidth()-3);
	lstItemInfo:AddItem(cLineBreakFour);
	containerHeight = containerHeight + cLineBreakFour:GetHeight();

	end -- End (Set)

	-- ITEM PLUGIN CONTROLS -------------------------------------------------------------------------------------------------------------
	if validID == true then
		local cControlsContainer = NewItemContainer(lstItemInfo:GetWidth()-3,20);

		local cOptions = Utils.Button();
		cOptions:SetParent(cControlsContainer);
		cOptions:SetSize(20,20);
		cOptions:SetLeft(8);
		cOptions:SetImageNormal(_IMAGES.BUTTONS.EDIT.NORMAL);
		cOptions:SetImageOver(_IMAGES.BUTTONS.EDIT.OVER);
		cOptions:SetToolTip(_TOOLTIPS[10][LANGID]);

		cOptions.Click = function ()
			if _itemInfo[1][LANGID] == nil or _itemInfo[1][LANGID] == "" then
				ItemInfo_EditPopup(itemID,_itemInfo[1][1]);
			else
				ItemInfo_EditPopup(itemID,_itemInfo[1][LANGID]);
			end
		end

		local cBuilds = Utils.Button();
		cBuilds:SetParent(cControlsContainer);
		cBuilds:SetSize(20,20);
		cBuilds:SetLeft(cOptions:GetLeft()+cOptions:GetWidth()+5);
		cBuilds:SetImageNormal(_IMAGES.BUTTONS.BUILD.NORMAL);
		cBuilds:SetImageOver(_IMAGES.BUTTONS.BUILD.OVER);
		cBuilds:SetToolTip(_TOOLTIPS[18][LANGID]);

		cBuilds.Click = function ()
			BuildsPopupMenu(itemID,_itemInfo[5]);
		end

		local cCompare = Utils.Button();
		cCompare:SetParent(cControlsContainer);
		cCompare:SetSize(20,20);
		cCompare:SetLeft(cBuilds:GetLeft()+cBuilds:GetWidth()+5);
		cCompare:SetImageNormal(_IMAGES.BUTTONS.COMPARE.NORMAL);
		cCompare:SetImageOver(_IMAGES.BUTTONS.COMPARE.OVER);
		cCompare:SetToolTip(_TOOLTIPS[12][LANGID]);

		cCompare.Click = function ()
			AddToCompare(itemID);
		end

		local cBookmark = Utils.Button();
		cBookmark:SetParent(cControlsContainer);
		cBookmark:SetSize(20,20);
		cBookmark:SetLeft(cCompare:GetLeft()+cCompare:GetWidth()+5);
		cBookmark:SetToolTip(_TOOLTIPS[13][LANGID]);

		UpdateBookmarkIcon(cBookmark,itemID);

		cBookmark.Click = function ()
			UpdateBookmark(cBookmark,itemID);
		end


		lstItemInfo:AddItem(cControlsContainer);
		containerHeight = containerHeight + cControlsContainer:GetHeight();

	else

		local cControlsContainer = NewItemContainer(lstItemInfo:GetWidth()-3,20);

		local cOptions = Utils.Button();
		cOptions:SetParent(cControlsContainer);
		cOptions:SetSize(20,20);
		cOptions:SetLeft(8);
		cOptions:SetImageNormal(_IMAGES.BUTTONS.EDIT.NORMAL);
		cOptions:SetImageOver(_IMAGES.BUTTONS.EDIT.OVER);
		cOptions:SetToolTip(_TOOLTIPS[10][LANGID]);

		cOptions.Click = function ()
			local contextMenu = Turbine.UI.ContextMenu();
			local menuItems = contextMenu:GetItems();
			local cLabel = Turbine.UI.MenuItem(_LABELS[106][LANGID],false);
			menuItems:Add(cLabel);
			local cEdit = Turbine.UI.MenuItem(_LABELS[107][LANGID]);
			cEdit.Click = function()
				printError("Dead item: " .. itemID);
			end
			menuItems:Add(cEdit);
			contextMenu:ShowMenu();
		end

		lstItemInfo:AddItem(cControlsContainer);
		containerHeight = containerHeight + cControlsContainer:GetHeight();

	end

	cItemContainer:SetHeight(containerHeight+20);
	lstItemInfo:SetHeight(containerHeight - lstItemInfo:GetTop());

	return cItemContainer;
end


function UpdateBookmark(control,itemID)
	if control == nil or itemID == nil then return end;
	local bookmarkID = GetBookmarkID(itemID);
	if bookmarkID == 0 then
		table.insert(_BOOKMARKS,itemID);
	else
		table.remove(_BOOKMARKS,bookmarkID);
	end
	UpdateBookmarkIcon(control,itemID);
end


function UpdateBookmarkIcon(control,itemID)
	if control == nil or itemID == nil then return end;
	if GetBookmarkID(itemID) ~= 0 then
		control:SetImageNormal(_IMAGES.BUTTONS.BOOKMARK_SELECTED.NORMAL);
		control:SetImageOver(_IMAGES.BUTTONS.BOOKMARK_SELECTED.OVER);
	else
		control:SetImageNormal(_IMAGES.BUTTONS.BOOKMARK_UNSELECTED.NORMAL);
		control:SetImageOver(_IMAGES.BUTTONS.BOOKMARK_UNSELECTED.OVER);
	end
end


function GetBookmarkID(itemID)
	if itemID == nil then return 0 end;
	local bookID = 0;
	for k,v in pairs (_BOOKMARKS) do
		if v == itemID then
			bookID = k;
			break;
		end
	end
	return bookID;
end


function ItemInfo_EditPopup(itemID,itemName)
	if itemID == nil then return end;

	local contextMenu = Turbine.UI.ContextMenu();
	local menuItems = contextMenu:GetItems();

	local cEdit = Turbine.UI.MenuItem(_LABELS[58][LANGID]);
	cEdit.Click = function()
		LoadItemData(itemID,itemName);
	end
	menuItems:Add(cEdit);

	local cSet = Turbine.UI.MenuItem();
	if _ITEMSDB[itemID][15] ~= nil then
		cSet:SetText(_LABELS[78][LANGID]);
		cSet.Click = function()
			Windows.set_EditSet(_ITEMSDB[itemID][15][2]);
		end
	else
		cSet:SetText(_LABELS[79][LANGID]);
		cSet.Click = function()
			Windows.set_EditSet(itemID);
		end
	end
	menuItems:Add(cSet);

	contextMenu:ShowMenu();
end


function BuildsPopupMenu(itemID,itemType)
	if itemID == nil or itemType==nil then return end;

	local contextMenu = Turbine.UI.ContextMenu();
	local menuItems = contextMenu:GetItems();

	local cLabel = Turbine.UI.MenuItem(_TOOLTIPS[18][LANGID],false);
	menuItems:Add(cLabel);

	for iKey,iVal in ipairs(_BUILDS) do
		if iVal[19] == nil then iVal[19] = "" end;
		local cBuildName = Turbine.UI.MenuItem(iVal[19]);
		cBuildName.Click = function()
			DisplaySlotsMenu(iKey,iVal[19],itemID,itemType);
		end
		menuItems:Add(cBuildName);
	end

	local cNewBuild = Turbine.UI.MenuItem("..".._LABELS[84][LANGID]);
	cNewBuild.Click = function()
		DisplaySlotsMenu(#_BUILDS+1,"..".._LABELS[84][LANGID],itemID,itemType);
	end
	menuItems:Add(cNewBuild);

	contextMenu:ShowMenu();

end


function DisplaySlotsMenu(buildID,buildName,itemID,itemType)
	if buildID == nil or buildName == nil or itemID == nil or itemType == nil then return end;
	local contextMenu = Turbine.UI.ContextMenu();
	local menuItems = contextMenu:GetItems();
	local _validSlots = LookupValidSlots(itemType);
	local cBuildName = Turbine.UI.MenuItem(buildName,false);
	menuItems:Add(cBuildName);
	for k,v in ipairs(_validSlots) do
		local cSlot = Turbine.UI.MenuItem(_TABLES.BUILDSLOTS[v][LANGID]);
		cSlot.Click = function()
			AddToBuild(buildID,itemID,v);
		end
		menuItems:Add(cSlot);
	end
	contextMenu:ShowMenu();
end


function LookupValidSlots(itemType)
	if itemType == nil then return end;
	local _slots = {};
	for k,v in ipairs(_TABLES.BUILDSLOTS) do
		if type(v.TYPE)=='table' then
			for key,value in pairs(v.TYPE) do
				if value == itemType then table.insert(_slots,k) break end;
			end
		else
			if v.TYPE == itemType then table.insert(_slots,k) end;
		end
	end
	return _slots;
end
