
-- This window displays the previous ten searches
function DrawSearchHistoryWin()

	wSearchHistory = Turbine.UI.Lotro.Window();
	wSearchHistory:SetSize(920,350);
	wSearchHistory:SetPosition((SCREENWIDTH/2)-(wSearchHistory:GetWidth()/2),(SCREENHEIGHT/2)-(wSearchHistory:GetHeight()/2));
	wSearchHistory:SetText(_LABELS[18][LANGID]);
	wSearchHistory:SetHideF12(true);
	wSearchHistory:SetCloseEsc(true);

	backSearchHistory = Turbine.UI.Control();
	backSearchHistory:SetParent(wSearchHistory);
	backSearchHistory:SetSize(920,350);
	backSearchHistory:SetPosition(0,-3);
	backSearchHistory:SetBackground(_IMAGES.BACKGROUNDS.SEARCH_HISTORY);
	backSearchHistory:SetBlendMode(4);
	backSearchHistory:SetMouseVisible(false);

	schHis_btnClear = Turbine.UI.Lotro.Button();
	schHis_btnClear:SetParent(wSearchHistory);
	schHis_btnClear:SetWidth(120);
	schHis_btnClear:SetPosition(wSearchHistory:GetWidth()-schHis_btnClear:GetWidth()-35,45);
	schHis_btnClear:SetText(_LABELS[4][LANGID]);
	schHis_btnClear:SetToolTip(_TOOLTIPS[9][LANGID]);

	schHis_btnClear.Click = function()
		ClearSearchHistory();
	end

	schHis_lstHistory = Turbine.UI.ListBox();
	schHis_lstHistory:SetParent(wSearchHistory);
	schHis_lstHistory:SetSize(wSearchHistory:GetWidth()-70,225);
	schHis_lstHistory:SetPosition((wSearchHistory:GetWidth()/2)-(schHis_lstHistory:GetWidth()/2),schHis_btnClear:GetTop()+35);
	schHis_sbHistory = NewScrollBar(schHis_lstHistory,"horizontal",wSearchHistory);

	schHis_lblHistory = NewWindowLabel(wSearchHistory,250,18,schHis_lstHistory:GetLeft(),schHis_btnClear:GetTop()+10,_LABELS[21][LANGID]);

	RefreshSearchHistoryList();

end


function RefreshSearchHistoryList()

	if wSearchHistory == nil then
		DrawSearchHistoryWin();
		return;
	end;

	schHis_lstHistory:ClearItems();

	for k,v in ipairs(_SEARCHHISTORY) do

		local cItemContainer = NewItemContainer(schHis_lstHistory:GetWidth(),22);

		local lblDetails = Turbine.UI.TextBox();
		lblDetails:SetParent(cItemContainer);
		lblDetails:SetHeight(cItemContainer:GetHeight());
		lblDetails:SetMarkupEnabled(true);
		lblDetails:SetMultiline(false);
		lblDetails:SetMouseVisible(false);
		lblDetails:SetSelectable(false);
		lblDetails:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
		lblDetails:SetFont(_FONTS[5]);
		lblDetails:SetForeColor(_COLORS[3]);
		lblDetails:SetText(GetHistoryText(v));
		Utils.AutoWidth(lblDetails);
		cItemContainer:SetWidth(lblDetails:GetWidth()+5);
		lblDetails:SetReadOnly(true);

		cItemContainer.MouseEnter = function()
			lblDetails:SetForeColor(_COLORS[7]);
		end

		cItemContainer.MouseLeave = function()
			lblDetails:SetForeColor(_COLORS[3]);
		end

		cItemContainer.MouseClick = function(sender,args)
			if args.Button == Turbine.UI.MouseButton.Right then
				HistoryMenuPopup(k);
			end
		end

		cItemContainer.MouseDoubleClick = function()
			PerformSearch(v);
			wSearchHistory:Close();
		end

		schHis_lstHistory:AddItem(cItemContainer);
	end

	HandleClearButtonEnab();

end


function HistoryMenuPopup(i)

	if i == nil then return end;

	local contextMenu = Turbine.UI.ContextMenu();
	local menuItems = contextMenu:GetItems();
	local cRemove = Turbine.UI.MenuItem(_LABELS[19][LANGID]);

	cRemove.Click = function()
		table.remove(_SEARCHHISTORY,i);
		RefreshSearchHistoryList();
	end

	menuItems:Add(cRemove);
	contextMenu:ShowMenu();
end


function GetHistoryText(t)

	if t == nil then return "" end;
	local rgb = "<rgb=#" .. Utils.HexTBColor(_COLORS[7]) .. ">";

	local sHistory = "";

	if t[1] ~= nil then sHistory = sHistory .. rgb .. _TABLES.ITEMINFO[1][LANGID] .. ":</rgb> " .. t[1] .. "  " end;		-- Item Name
	if t[2] ~= nil and t[2].Min ~= nil then sHistory = sHistory .. rgb .. _LABELS[6][LANGID] .. ":</rgb> " .. tostring(t[2].Min) .. "  " end;		-- Min Level
	if t[2] ~= nil and t[2].Max ~= nil then sHistory = sHistory .. rgb .. _LABELS[7][LANGID] .. ":</rgb> " .. tostring(t[2].Max) .. "  " end;		-- Max Level
	if t[14] ~= nil then sHistory = sHistory .. rgb .. _TABLES.ITEMINFO[14][LANGID] .. ":</rgb> " .. _TABLES.CLASS[t[14]][LANGID] .. "  " end;		-- Class
	if t[7] ~= nil then sHistory = sHistory .. rgb .. _TABLES.ITEMINFO[7][LANGID] .. ":</rgb> " .. _TABLES.ARMORTYPE[t[7]][LANGID] .. "  " end;		-- Armour Type
	if t[13] ~= nil then sHistory = sHistory .. rgb .. _TABLES.ITEMINFO[13][LANGID] .. ":</rgb> " .. _TABLES.DAMAGETYPE[t[13]][LANGID] .. "  " end;		-- Damage Type

	if t[9] ~= nil and type(t[9]) == 'table' then		-- Weapon/Shield Type
		sHistory = sHistory .. rgb .. _TABLES.ITEMINFO[9][LANGID] .. ":</rgb> "
		for k,v in pairs(t[9]) do
			sHistory = sHistory .. "(" .. _TABLES.WEAPONTYPE[k][LANGID] .. ") ";
		end
		sHistory = sHistory .. " ";
	end

	if t[5] ~= nil and type(t[5]) == 'table' then		-- Item Type
		sHistory = sHistory .. rgb .. _LABELS[8][LANGID] .. ":</rgb> "
		for k,v in pairs(t[5]) do
			sHistory = sHistory .. "(" .. _TABLES.SLOT[k][LANGID] .. ") ";
		end
		sHistory = sHistory .. " ";
	end

	if t[6] ~= nil and type(t[6]) == 'table' then		-- Stats
		sHistory = sHistory .. rgb .. _TABLES.ITEMINFO[6][LANGID] .. ":</rgb> "
		for k,v in pairs(t[6]) do
			sHistory = sHistory .. "(" .. _TABLES.STATS[k][LANGID] .. ") ";
		end
		sHistory = sHistory .. " ";
	end

	if t[4] ~= nil and type(t[4]) == 'table' then		-- Quality
		sHistory = sHistory .. rgb .. _TABLES.ITEMINFO[4][LANGID] .. ":</rgb> "
		for k,v in pairs(t[4]) do
			sHistory = sHistory .. "(" .. _TABLES.QUALITY[k][LANGID] .. ") ";
		end
		sHistory = sHistory .. " ";
	end

	return sHistory;

end


function HandleClearButtonEnab()

	if schHis_lstHistory:GetItemCount() > 0 then
		schHis_btnClear:SetEnabled(true);
	else
		schHis_btnClear:SetEnabled(false);
		search_btnHistory:SetEnabled(false);
	end

end


function ClearSearchHistory()
	local tempMsg = Utils.MessageBox(_LABELS[20][LANGID],"MBYESNO");
	tempMsg.ButtonClick = function (sender,args)
		if args.Button == "MBYES" then
			Utils.ClearTable(_SEARCHHISTORY);
			RefreshSearchHistoryList();
		end
	end
end


function DisplaySearchHistoryWin()
	if wSearchHistory == nil then DrawSearchHistoryWin() end;
	wSearchHistory:SetVisible(true);
	wSearchHistory:Activate();
end
